#include "stdlib.h"
#include "SDL.h"
#include "SDL_image.h"
#include "ScoreScreen.h"
#include "SoundManager.h"
#include "Sheep.h"
#include "Herd.h"
#include "joystickmapping.h"

class Game {
private:
    Herd *herd;
    unsigned int escapeTime;
    unsigned int tranquilizerFired;
    SDL_Surface *bg;
    SDL_Surface *screen;
    AWSprite *penaltyFlag;
    SoundManager *soundManager;
    AWSplashScreen *instructions;
    AWSplashScreen *credits;

public:
    Game(SDL_Surface *sc);
    ~Game();
    void draw_grass();
    int wait_a_bit(bool showPenalty);
    unsigned int play_one_escapee();
    unsigned int play_one_game();
    void show_instructions();
    void show_credits();
    void play_SheepDash();
};



Game::Game(SDL_Surface *sc) {
    soundManager = new SoundManager();
    herd = new Herd(11, soundManager);
    screen = sc;
    // Do background
    SDL_Surface* temp = IMG_Load("grass.jpg");
    bg = SDL_DisplayFormat(temp);
    SDL_FreeSurface(temp);
    penaltyFlag = new AWSprite("penalty.jpg", 1);
    penaltyFlag->set_transparent_colour(0, 0, 0);
    penaltyFlag->set_world_position(60, 40);
    instructions = new AWSplashScreen("instructions.bmp");
    credits = new AWSplashScreen("credits.bmp");
}

void Game::draw_grass() {
    SDL_BlitSurface(bg, NULL, screen, NULL);
}

unsigned int Game::play_one_escapee() {             // returns the reaction time in milliseconds
    unsigned int startTime = SDL_GetTicks();
    unsigned int currentTime;
    unsigned int timeTaken = PENALTYTIME;
    bool done = false;
    unsigned int waitFor = 1600 + (rand() % 1500);
    bool weHaveARunner = false;
    bool gotHim = false;
    bool quitPressed = false;
    bool firing = false;
    SDL_Event event;

    startTime = SDL_GetTicks();
    while(!done && !quitPressed) {
        draw_grass();
        herd->update();
        SDL_Flip(screen);
        currentTime = SDL_GetTicks();
        if(!weHaveARunner && ((currentTime - startTime) > waitFor)) {
            herd->send_a_runner();
            escapeTime = currentTime;
            weHaveARunner = true;
        }
        if(SDL_PollEvent(&event)) {
            switch (event.type) {
                case SDL_QUIT:
                    quitPressed = true;
                    break;
                case SDL_KEYDOWN:
                    switch (event.key.keysym.sym) {
                        case SDLK_ESCAPE:
                        case SDLK_q:
                            quitPressed = true;
                            break;
                        case SDLK_SPACE:
                            firing = true;
                            break;
                    }
                    break;
                case SDL_JOYBUTTONDOWN:  /* Joystick Button Press */
                    switch (event.jbutton.button) {
                        case GP2X_BUTTON_L:
                        case GP2X_BUTTON_R:
                            firing = true;
                            break;
                        case GP2X_BUTTON_SELECT:
                            quitPressed = true;
                            break;
                        case GP2X_BUTTON_START:
                            // showHelp = true;
                            break;
                    }
                    break;
                case SDL_JOYBUTTONUP:  /* Joystick Button Release */
                    switch (event.jbutton.button) {
                        case GP2X_BUTTON_L:
                        case GP2X_BUTTON_R:
                            firing = false;
                            break;
                        case GP2X_BUTTON_START:
                            // showHelp = false;
                            break;
                    }
                    break;
            }
        }

        if(firing) {
            if(weHaveARunner) {
                timeTaken = SDL_GetTicks() - escapeTime;
                herd->runner_was_hit();
                gotHim = true;
            } else {
                gotHim = false;
				timeTaken = PENALTYTIME;
            }
            done = true;
        } else {
            if(weHaveARunner && (herd->get_running_sheep() == NULL)) {
                // He ran off the screen before we fired...
                gotHim = false;
				timeTaken = GOTAWAYTIME;
                herd->runner_got_away();
                done = true;
            }
        }
    }

    if(quitPressed) timeTaken = PLAYERWANTSTOQUIT;

    return timeTaken;
}


int Game::wait_a_bit(bool showPenalty) {             // this just lets the animations run for a bit
    unsigned int startTime = SDL_GetTicks();
    unsigned int currentTime;
    unsigned int waitFor = (1500);
    bool quitPressed = false;
    bool finishedWaiting = false;
    SDL_Event event;

    startTime = SDL_GetTicks();
    while(!quitPressed && !finishedWaiting) {
        draw_grass();
        herd->update();
        if(showPenalty) {
            penaltyFlag->update_everything();
        }
        SDL_Flip(screen);
        currentTime = SDL_GetTicks();
        if(currentTime > startTime + waitFor) finishedWaiting = true;
        if(SDL_PollEvent(&event)) {
            switch (event.type) {
                case SDL_QUIT:
                    quitPressed = true;
                    break;
                case SDL_KEYDOWN:
                    switch (event.key.keysym.sym) {
                        case SDLK_ESCAPE:
                        case SDLK_q:
                            quitPressed = true;
                            break;
                    }
                    break;
                case SDL_JOYBUTTONDOWN:  /* Joystick Button Press */
                    switch (event.jbutton.button) {
                        case GP2X_BUTTON_SELECT:
                            quitPressed = true;
                            break;
                    }
                    break;
            }
        }
    }
    if(quitPressed) return PLAYERWANTSTOQUIT;
    else return 0;
}


unsigned int Game::play_one_game() {
    int r;
    unsigned int totalTime = 0;
    unsigned int thisTime;
    unsigned int waitResult;
    unsigned int times[HOWMANYRUNNERS];

    herd->reset();
    for(r = 0; r < HOWMANYRUNNERS; r++) {
        thisTime = this->play_one_escapee();
        if(thisTime == PLAYERWANTSTOQUIT) return PLAYERWANTSTOQUIT;
        times[r] = thisTime;
        totalTime += thisTime;
        waitResult = this->wait_a_bit(thisTime == PENALTYTIME);
        if(waitResult == PLAYERWANTSTOQUIT) return PLAYERWANTSTOQUIT;
    }
    ScoreScreen *scoreScreen = new ScoreScreen("Pulse.bmp", times);
    scoreScreen->display(0);
    delete scoreScreen;

    return totalTime / HOWMANYRUNNERS;
}

void Game::show_instructions() {
    instructions->display(0);
}

void Game::show_credits() {
    credits->display(0);
}

// Keep playing until the player quits!
void Game::play_SheepDash() {
    int t;

    show_instructions();
    do {
        t = play_one_game();
    } while(t != PLAYERWANTSTOQUIT);

    show_credits();
}



