#ifndef         SHEEP_DOT_H
#define         SHEEP_DOT_H         (1)
#include "stdlib.h"
#include "sheepdash.h"
#include "AWSprite.h"
#include "SoundManager.h"

class Sheep {
private:
    AWSprite *eating;
    AWSprite *running;
    AWSprite *sleeping;
    AWSprite *currentState;
    int state;
    SoundManager *soundManager;

public:
    Sheep(int yPos, SoundManager *sM);
    ~Sheep();

    void make_a_run_for_it();
    void go_to_sleep();
    void update();
    void set_state(int newState);
    int get_state();
    void reset(int yPos);
    int get_x();
    int get_y();
};

Sheep::Sheep(int yPos, SoundManager *sM) {
    soundManager = sM;

    eating = new AWSprite("eating.bmp", 15);
    eating->set_currentFrame(rand() % 15);
    eating->set_transparent_colour(106, 76, 48);
    eating->set_auto_animate(70 + (rand() % 90));

    running = new AWSprite("running.bmp", 9);
    running->set_transparent_colour(106, 76, 48);
    running->set_visibility(false);

    sleeping = new AWSprite("sleeping.bmp", 7);
    sleeping->set_animation_subset(6, 6);
    sleeping->set_auto_animate(0);
    sleeping->set_transparent_colour(106, 76, 48);
    sleeping->set_visibility(false);

    this->reset(yPos);
}

Sheep::~Sheep() {
}

void Sheep::reset(int yPos) {
    state = EATING;
    currentState = eating;
    currentState->set_visibility(true);
    currentState->set_world_position((rand()%10), yPos);
}

// This tells the Sheep to run for it
void Sheep::make_a_run_for_it() {
    if(currentState != eating) return;      // Should never happen

    running->set_world_position(eating->get_x(), eating->get_y());
    running->set_auto_animate(10);
    running->set_accelerations(1.6f, 0.0f);
    running->set_auto_accelerate(20);
    #ifdef GP2X
    running->set_velocities(10.0f, 0.0f);
    #else
    running->set_velocities(13.0f, 0.0f);
    #endif

    running->set_auto_move(20);

    eating->set_visibility(false);
    running->set_visibility(true);

    state = RUNNING;
    currentState = running;
}

// This gets called when the sheep has been stopped in its tracks
void Sheep::go_to_sleep() {
    if(currentState != running) return;     // Should never happen

    sleeping->set_animation_subset(0, 6);
    sleeping->set_auto_animate(130, 1);
    sleeping->set_world_position(running->get_x(), running->get_y());

    running->set_visibility(false);
    sleeping->set_visibility(true);

    state = SLEEPING;
    currentState = sleeping;
}


void Sheep::update() {
    currentState->update_everything(0, 0);
    if(currentState == running) { // Make sure we don't run off the screen!
        if(currentState->get_x() >= 320) {
            currentState->set_velocities(0.0f, 0.0f);
            currentState->set_auto_move(0);
            currentState->set_visibility(false);
            currentState->set_world_position(0, 0);
            state = GOTAWAY;
        }
    } else if(currentState == eating) {
        // bleat every now and again
        #ifdef GP2X
        int r = rand() % 6000;
        #else
        int r = rand() % 50000;
        #endif
        switch(r) {
            case 0:
                soundManager->play(BLEAT3);
                break;
            case 1:
                soundManager->play(BLEAT4);
                break;
            case 2:
                soundManager->play(BLEAT5);
                break;
        }
    }
}


void Sheep::set_state(int newState) {
    state = newState;
}

int Sheep::get_state() {
    return state;
}

int Sheep::get_x() {
    if(currentState == NULL) return 0;
    else return currentState->get_x();
}

int Sheep::get_y() {
    if(currentState == NULL) return 0;
    else return currentState->get_y();
}


#endif


